# All biotypes specified here will be processed and shown in the final visualization in the same specified order
# Repetitions are accepted
BIOTYPES = [
    'protein_coding',
    'lincRNA',
    #         '3prime_overlapping_ncrna',
    #         'antisense',
    #         'IG_C_gene',
    #         'IG_C_pseudogene',
    #         'IG_D_gene',
    #         'IG_J_gene',
    #         'IG_J_pseudogene',
    #         'IG_V_gene',
    #         'IG_V_pseudogene',
    #         'known_ncrna',
    #         'miRNA',
    #         'misc_RNA',
    #         'Mt_rRNA',
    #         'Mt_tRNA',
    #         'non_coding',
    #         'polymorphic_pseudogene',
    #         'processed_pseudogene',
    #         'processed_transcript',
    #         'pseudogene',
    #         'rRNA',
    #         'sense_intronic',
    #         'sense_overlapping',
    #         'snoRNA',
    #         'snRNA',
    #         'TEC',
    #         'transcribed_processed_pseudogene',
    #         'transcribed_unitary_pseudogene',
    #         'transcribed_unprocessed_pseudogene',
    #         'translated_processed_pseudogene',
    #         'translated_unprocessed_pseudogene',
    #         'TR_C_gene',
    #         'TR_D_gene',
    #         'TR_J_gene',
    #         'TR_J_pseudogene',
    #         'TR_V_gene',
    #         'TR_V_pseudogene',
    #         'unitary_pseudogene',
    #         'unprocessed_pseudogene',
]
